@@ -18,6 +18,7 @@ import argparse
 import collections
 import importlib
 import os.path
+import logging.config
 
 import babel.dates
 import babel.support
@@ -29,6 +30,8 @@ import redis as redispy
 import sqlalchemy
 import yaml
 
+from raven import Client
+from raven.middleware import Sentry
 from webassets import Environment as AssetsEnvironment
 from werkzeug.exceptions import HTTPException
 from werkzeug.routing import Map
@@ -42,20 +45,20 @@ from warehouse.middleware import PoweredBy
 from warehouse.packaging import helpers as packaging_helpers
 from warehouse.utils import AttributeDict, merge_dict, convert_to_attr_dict
 
+from warehouse.accounts.urls import urls as accounts_urls
+from warehouse.packaging.urls import urls as packaging_urls
+from warehouse.legacy.urls import urls as legacy_urls
+
 
 class Warehouse(object):
 
     metadata = sqlalchemy.MetaData()
 
     model_names = {
+        "accounts": "warehouse.accounts.models:Model",
         "packaging": "warehouse.packaging.models:Model",
     }
 
-    url_names = [
-        "warehouse.ui.urls",
-        "warehouse.legacy.urls",
-    ]
-
     def __init__(self, config, engine=None, redis=None):
         self.config = convert_to_attr_dict(config)
 
@@ -81,12 +84,8 @@ class Warehouse(object):
                 self.redis,
             )
 
-        # Setup our URL routing
-        url_rules = []
-        for name in self.url_names:
-            mod = importlib.import_module(name)
-            url_rules.extend(getattr(mod, "__urls__"))
-        self.urls = Map(url_rules)
+        # Set up our URL routing
+        self.urls = Map(accounts_urls + packaging_urls + legacy_urls)
 
         # Initialize our Translations engine
         self.trans = babel.support.NullTranslations()
@@ -105,7 +104,6 @@ class Warehouse(object):
         # Install Babel
         self.templates.filters.update({
             "package_type_display": packaging_helpers.package_type_display,
-
             "format_date": babel.dates.format_date,
             "format_datetime": babel.dates.format_datetime,
             "format_time": babel.dates.format_time,
@@ -142,6 +140,9 @@ class Warehouse(object):
             self.config.security.csp,
         )
 
+        if "sentry" in self.config:
+            self.wsgi_app = Sentry(self.wsgi_app, Client(**self.config.sentry))
+
         # Serve the static files if we're in debug
         if self.config.debug:
             self.wsgi_app = SharedDataMiddleware(
@@ -153,6 +154,9 @@ class Warehouse(object):
                 {"/static/": self.config.assets.directory},
             )
 
+        # configure logging
+        logging.config.dictConfig(self.config.logging)
+
     def __call__(self, environ, start_response):
         """
         Shortcut for :attr:`wsgi_app`.
