@@ -18,18 +18,21 @@ import collections
 import datetime
 import os.path
 import urlparse
+import logging
 
 from collections import namedtuple
 
 from sqlalchemy.sql import and_, select, func
 
 from warehouse import models
+from warehouse.accounts.tables import users, emails
 from warehouse.packaging.tables import ReleaseDependencyKind
 from warehouse.packaging.tables import (
     packages, releases, release_files, description_urls, journals,
-    classifiers, release_classifiers, release_dependencies,
+    classifiers, release_classifiers, release_dependencies, roles
 )
 
+log = logging.getLogger(__name__)
 
 Project = namedtuple("Project", ["name"])
 
@@ -60,6 +63,47 @@ class Model(models.Model):
             if result is not None:
                 return Project(result)
 
+    def get_projects_for_user(self, username):
+        query = """
+            SELECT DISTINCT ON (lower(name)) name, summary
+            FROM (
+                SELECT package_name
+                FROM roles
+                WHERE user_name = %(username)s
+            ) roles
+            INNER JOIN (
+                SELECT name, summary
+                FROM releases
+                ORDER BY _pypi_ordering DESC
+            ) releases
+            ON (releases.name = roles.package_name)
+            ORDER BY lower(name);
+        """
+
+        with self.engine.connect() as conn:
+            return [dict(r) for r in conn.execute(query, username=username)]
+
+    def get_users_for_project(self, project):
+        query = (
+            select(
+                [users.c.username, emails.c.email],
+                from_obj=users.outerjoin(
+                    emails, emails.c.user_id == users.c.id,
+                ),
+            )
+            .where(and_(
+                users.c.username == roles.c.user_name,
+                roles.c.package_name == project,
+            ))
+            .order_by(
+                roles.c.role_name.desc(),
+                func.lower(roles.c.user_name),
+            )
+        )
+
+        with self.engine.connect() as conn:
+            return [dict(r) for r in conn.execute(query)]
+
     def get_hosting_mode(self, name):
         query = (
             select([packages.c.hosting_mode])
@@ -202,6 +246,13 @@ class Model(models.Model):
                     result["name"],
                     result["filename"],
                 )
+                if not os.path.exists(result["filepath"]):
+                    log.error(
+                        "%s missing for package %s %s",
+                        result["filepath"],
+                        result["name"],
+                        result["version"])
+                    continue
                 result["url"] = "/".join([
                     "/packages",
                     result["python_version"],
